# -*- coding: utf-8 -*-

import datetime as dt
import mutagen
import mutagen.id3
import mutagen.mp4


class Track:

    def __init__(self, path):
        self._mutagen = mutagen.File(filename=path)

    @property
    def path(self):
        return self._mutagen.filename

    @property
    def has_tags(self):
        return self._mutagen and self._mutagen.tags

    @property
    def has_id3_tags(self):
        return self.has_tags and isinstance(self._mutagen.tags, mutagen.id3.ID3)

    @property
    def has_mp4_tags(self):
        return self.has_tags and isinstance(self._mutagen.tags, mutagen.mp4.MP4Tags)

    def register_play_count(self, player, library_id, register_dt, play_count, tag_set_cb=None):
        assert isinstance(register_dt, dt.datetime), register_dt
        assert isinstance(play_count, int), play_count
        tag_label = 'symuid:pcnt:{}:{}:{:.0f}'.format(
            player, library_id, register_dt.timestamp(),
        )
        if self.has_id3_tags:
            tag_label_id3 = 'TXXX:' + tag_label
            if tag_label_id3 in self._mutagen.tags:
                reg_data = self._mutagen.tags[tag_label_id3].text
                assert len(reg_data) == 1, reg_data
                assert int(reg_data[0]) == play_count, (reg_data, play_count)
            else:
                # mutagen.id3._specs.EncodedTextSpec.write encodes
                # 'desc' and 'text'
                tag = mutagen.id3.TXXX(
                    encoding=mutagen.id3.Encoding.LATIN1,
                    desc=tag_label,
                    text=[str(play_count)],
                )
                self._mutagen.tags.add(tag)
                self._mutagen.save()
                if tag_set_cb:
                    tag_set_cb(self, tag)
        elif self.has_mp4_tags:
            tag_label_mp4 = '----:' + tag_label
            if tag_label_mp4 in self._mutagen.tags:
                reg_data = self._mutagen.tags[tag_label_mp4]
                assert len(reg_data) == 1, reg_data
                reg_count = int.from_bytes(
                    bytes=reg_data[0],
                    byteorder='big',
                    signed=True,
                )
                assert reg_count == play_count, (reg_count, play_count)
            else:
                self._mutagen.tags[tag_label_mp4] = tag = mutagen.mp4.MP4FreeForm(
                    # "a signed big-endian integer with length one of { 1,2,3,4,8 } bytes"
                    # TODO set byte length properly
                    # setting signed=True just to be explicit here
                    #  (irrelevant for positive integers)
                    data=play_count.to_bytes(1, byteorder='big', signed=True),
                    dataformat=mutagen.mp4.AtomDataType.INTEGER,
                )
                self._mutagen.save()
                if tag_set_cb:
                    tag_set_cb(self, tag)
        else:
            raise NotImplementedError()
